from __future__ import annotations  # Allows type hinting of own class (default in Python 4.0)

import string
import typing
import dataclasses

from manganelo.api import APIBase


@dataclasses.dataclass
class MangaSearchResult:
	title: str
	url: str


class MangaSearch(list, APIBase):
	def __init__(self, title: str) -> None:
		super().__init__()

		self._url = self.SEARCH_URL + self._format_title(title)

		self._page_soup = self._get_soup()

		self._get_results()

	def __enter__(self) -> MangaSearch:
		return self

	def __exit__(self, exc_type, exc_val, exc_tb) -> None:
		""" Context manager exit method """

	def _get_results(self) -> typing.Iterable[MangaSearchResult]:
		panels = self._page_soup.find(class_="panel-search-story")

		stories = panels.find_all(class_="search-story-item")

		# Iterate through the soup and extract the information
		for i, ele in enumerate(stories):
			item = ele.find(class_="item-img")

			title = item["title"]
			link = item["href"]

			manga_result = MangaSearchResult(title=title, url=link)

			self.append(manga_result)

	def _format_title(self, title: str) -> str:
		allowed_characters = string.ascii_letters + string.digits + "_"\

		# Remove all characters which are not allowed and replace spaces with underscores
		return "".join([char.lower() for char in title.replace(" ", "_") if char in allowed_characters])



from __future__ import annotations  # Allows type hinting of own class (default in Python 4.0)

import typing

from manganelo.api import APIBase


class MangaInfo(APIBase):
	title: str
	authors: typing.List[str]
	genres: typing.List[str]
	alt_titles: typing.List[str]
	status: str
	url: str

	def __init__(self, url: str):
		super().__init__()

		self._url = url

		self._values = {}

		self._page_soup = self._get_soup()

		self._get_results()

	def __str__(self) -> str:
		return self._url

	def __enter__(self) -> MangaInfo:
		return self

	def __exit__(self, exc_type, exc_val, exc_tb):
		""" Context manager exit point """

	def _get_results(self) -> MangaInfoDC:
		info_panel = self._page_soup.find(class_="panel-story-info")

		story_info_right = info_panel.find(class_="story-info-right")
		table_section = story_info_right.find(class_="variations-tableInfo")

		table_values = table_section.find_all(class_="table-value")

		self._values = {
			"title": story_info_right.find("h1").text,
			"authors": [author.strip() for author in table_values[1].text.split("-")],
			"genres": [genre.strip() for genre in table_values[3].text.split("-")],
			"alt_titles": table_values[0].text,
			"status": table_values[2].text,
			"url": self._url
		}

		for k, v in self._values.items():
			setattr(self, k, v)


from __future__ import annotations  # Allows type hinting of own class (default in Python 4.0)

import ast
import typing
import dataclasses

from manganelo.api import APIBase

@dataclasses.dataclass
class MangaChapter:
	url: str
	chapter_num: float


class ChapterList(list, APIBase):
	def __init__(self, url: str) -> None:
		super().__init__()

		self._url = url

		self._page_soup = self._get_soup()

		self._get_results()

	def __enter__(self):
		return self

	def __exit__(self, exc_type, exc_val, exc_tb):
		""" Context manager exit point """

	def _get_results(self) -> typing.Iterable[MangaChapter]:
		panels = self._page_soup.find(class_="panel-story-chapter-list")

		chapters_soup = panels.find_all(class_="a-h")

		for i, ele in enumerate(reversed(chapters_soup)):
			url = ele.find("a")["href"]

			chapter_num = ast.literal_eval(url.split("chapter_")[-1])

			chapter = MangaChapter(url=url, chapter_num=chapter_num)

			self.append(chapter)


import requests
import typing

from bs4 import BeautifulSoup


class APIBase:
	SEARCH_URL = "http://manganelo.com/search/"

	def _get_soup(self) -> typing.Union[BeautifulSoup, None]:
		default_headers = requests.utils.default_headers()

		r = requests.get(self._url, stream=True, timeout=5, headers=default_headers)

		r.raise_for_status()

		return BeautifulSoup(r.content, "html.parser") if r.status_code == requests.codes.ok else None



from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
  name="manganelo",
  packages=find_packages(),
  version="1.1",
  license="MIT",

  long_description_content_type="text/markdown",

  description="Unofficial API for the Manganelo (and Mangakakalot) website.",
  long_description=long_description,

  author="Joshua Nixon",
  author_email="nixonjoshua98@gmail.com",

  url="https://github.com/nixonjoshua98/manganelo",

  download_url="https://github.com/nixonjoshua98/manganelo/archive/v1.0.tar.gz",
  
  keywords=["manga", "manganelo", "scrapper", "web", "mangakakalot"],
  
  install_requires=[
    "bs4",
    "requests",
  ],

  classifiers=[
    "Programming Language :: Python :: 3",
    "License :: OSI Approved :: MIT License",
    "Operating System :: OS Independent",
    "Development Status :: 5 - Production/Stable",
    "Intended Audience :: Developers",  
    "Topic :: Software Development :: Build Tools",
  ],

  python_requires='>=3.7'
)
